#ifdef _WIN32
	#include <windows.h>
	#include <shlobj.h>
#endif

#include "worldb.h"
#include "circuit8.cpp"
#include "pill.cpp"

WorldB::WorldB()
{
	// settings and pointers
	settings = Settings::Instance();
		retinasperrow = settings->getCVarPtr("retinasperrow");
		critter_maxlifetime = settings->getCVarPtr("critter_maxlifetime");
		critter_maxenergy = settings->getCVarPtr("critter_maxenergy");
		critter_autosaveinterval = settings->getCVarPtr("critter_autosaveinterval");
		critter_autoexchangeinterval = settings->getCVarPtr("critter_autoexchangeinterval");
		critter_killhalfat = settings->getCVarPtr("critter_killhalfat");
		critter_retinasize = settings->getCVarPtr("critter_retinasize");
		critter_sightrange = settings->getCVarPtr("critter_sightrange");
		critter_raycastvision = settings->getCVarPtr("critter_raycastvision");

		food_maxlifetime = settings->getCVarPtr("food_maxlifetime");
		food_maxenergy = settings->getCVarPtr("food_maxenergy");
		energy = settings->getCVarPtr("energy");

		pill_maxlifetime = settings->getCVarPtr("pill_maxlifetime");
		pill_energylevel = settings->getCVarPtr("pill_energylevel");
		pill_maxtrip = settings->getCVarPtr("pill_maxtrip");

		condensed_colourmotors = settings->getCVarPtr("condensed_colourmotors");

	statsBuffer = Statsbuffer::Instance();
	critterselection = Critterselection::Instance();

	freeEnergy = *food_maxenergy * *energy;
		
	currentCritterID	= 1;
	insertCritterCounter	= 0;
	autosaveCounter		= 0.0f;
	autoexchangeCounter	= 0.0f;
	insertHight		= 1.0f;
	
	// vision retina allocation
	items = 4 * 800 * 600;
	retina = (unsigned char*)malloc(items);
	memset(retina, 0, items);

	// home & program directory
	createDirs();

	m_collisionConfiguration = new btDefaultCollisionConfiguration();

	m_dispatcher = new btCollisionDispatcher(m_collisionConfiguration);
	//m_dispatcher = new SpuGatheringCollisionDispatcher(m_collisionConfiguration);

	btVector3 worldAabbMin(-10000,-10000,-10000);
	btVector3 worldAabbMax(10000,10000,10000);
	m_broadphase = new btAxisSweep3 (worldAabbMin, worldAabbMax);
	m_broadphase->getOverlappingPairCache()->setInternalGhostPairCallback(new btGhostPairCallback());
	m_solver = new btSequentialImpulseConstraintSolver;

	m_dynamicsWorld = new btDiscreteDynamicsWorld(m_dispatcher,m_broadphase,m_solver,m_collisionConfiguration);
	
// 	btVector3 v = m_dynamicsWorld->getGravity();
// 	cerr << v.y() << endl;
// 	m_dynamicsWorld->setGravity( btVector3(0.0f, -50.0f, 0.0f) );

// 	m_dynamicsWorld->getSolverInfo().m_solverMode = SOLVER_USE_WARMSTARTING + SOLVER_SIMD;
	m_dynamicsWorld->getSolverInfo().m_solverMode = SOLVER_SIMD + SOLVER_USE_WARMSTARTING + SOLVER_CACHE_FRIENDLY;
	
	m_dynamicsWorld->getSolverInfo().m_numIterations = 10;
	// raycast
	raycast = new Raycast(m_dynamicsWorld);

	// mousepicker
	mousepicker = new Mousepicker(m_dynamicsWorld);

	// determine vision width
	picwidth = *retinasperrow * (*critter_retinasize+1);

	// reset cam
		resetCamera();

	
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawWireframe|btIDebugDraw::DBG_DrawConstraints+btIDebugDraw::DBG_DrawConstraintLimits);
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawConstraints+btIDebugDraw::DBG_DrawConstraintLimits);
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawConstraints);
// 	debugDrawer.setDebugMode(btIDebugDraw::DBG_DrawConstraintLimits);
// 	m_dynamicsWorld->setDebugDrawer(&debugDrawer);

}

void WorldB::init()
{
	makeFloor();

// 	// reset cam
// 		resetCamera();

	if ( settings->getCVar("autoload") )
		loadAllCritters();
}

void WorldB::castMouseRay()
{
// 	cerr << "casting" << endl;
	mouseRay = raycast->cast( camera.position.getOrigin(), mouseRayTo );

	mouseRayHit = false;
	if ( mouseRay.hit )
	{
// 		if ( !( mouseRay.hitBody->isStaticObject() || mouseRay.hitBody->isKinematicObject() ) )
// 		{
			Entity* e = static_cast<Entity*>(mouseRay.hitBody->getUserPointer());
			if ( e->type == 1 || e->type == 0 || e->type == 3)
			{
				mouseRayHit = true;
				mouseRayHitEntity = e;
			}
// 		}
	}
}

void WorldB::calcMouseDirection()
{
// 	cerr << "updating mouserayto" << endl;
	mouseRayTo = camera.getScreenDirection(*mousex, *mousey);
}

void WorldB::moveInMouseDirection(bool towards)
{
// 	cerr << "updating mouserayto" << endl;
	if ( towards )
		camera.moveTowards(mouseRayTo.normalized());
	else
		camera.moveAwayFrom(mouseRayTo.normalized());
}

void WorldB::pickBody(const int& x, const int& y)
{
	if ( mouseRayHit )
	{
		if ( mouseRayHitEntity->type == 0 || mouseRayHitEntity->type == 1 || mouseRayHitEntity->type == 3)
		{
			btRigidBody* b = static_cast<btRigidBody*>(mouseRay.hitBody);

			// if critter, and it's the head's ghostobject we touch, overwrite with head's body
			if ( mouseRayHitEntity->type == 0 )
			{
				btCollisionObject* co = static_cast<btCollisionObject*>(mouseRay.hitBody);
				CritterB* c = static_cast<CritterB*>(mouseRayHitEntity);
				if ( co == c->body.mouths[0]->ghostObject )
					b = c->body.mouths[0]->body;
			}
			
			mousepicker->attach( b, mouseRay.hitPosition, camera.position.getOrigin(), mouseRayTo );
			mousepicker->pickedBool = &mouseRayHitEntity->isPicked;
			*mousepicker->pickedBool = true;
		}
	}
}

void WorldB::selectBody(const int& x, const int& y)
{
	if ( mouseRayHit )
		if ( mouseRayHitEntity->type == 0 )
			critterselection->registerCritter(static_cast<CritterB*>(mouseRayHitEntity));
}

void WorldB::movePickedBodyTo()
{
	if ( mousepicker->active )
		mousepicker->moveTo( camera.position.getOrigin(), mouseRayTo );
}

void WorldB::movePickedBodyFrom()
{
	if ( mousepicker->active )
		mousepicker->moveFrom( camera.position.getOrigin() );
}

void WorldB::process()
{
//		cout << "process starts" << endl;

//		circuit8.screenidx = 0; // non-animated psychic grid or individual grids
//		circuit8.screenidx += 1; if (circuit8.screenidx > 63) circuit8.screenidx = 0; // one-way animated psychic grid

		if (circuit8.screendirection == 1) { circuit8.screenidx += 1;  } else if (circuit8.screendirection == 0) { circuit8.screenidx -= 1; }
		if (circuit8.screenidx == 1024 - 1) { circuit8.screendirection = 0; }
		if (circuit8.screenidx == 0)  { circuit8.screendirection = 1; } // look out, back-and-forth animated psychic grid with scroll for space bonding
		if (circuit8.upscrolls < 4 && circuit8.screenidx == 0) { // is it better or worse to do a number of scroll-ups the height is divisible by? going with not for now
			float temprowr[64];
			float temprowg[64];
			float temprowb[64];
			for (int tempidx = 0; tempidx < 1024; tempidx++) {
				for (int tx = 0; tx < 64; tx++) {
					temprowr[tx] = circuit8.screen.screen[tx][1].r[tempidx]; // scrolls up
					temprowg[tx] = circuit8.screen.screen[tx][1].g[tempidx]; // scrolls up
					temprowb[tx] = circuit8.screen.screen[tx][1].b[tempidx]; // scrolls up
				}
				for (int tx = 0; tx < 64; tx++) {
					for (int ty = 0; ty < 48 - 1; ty++) {
						circuit8.screen.screen[tx][ty].r[tempidx] = circuit8.screen.screen[tx][ty+1].r[tempidx]; // scrolls up
						circuit8.screen.screen[tx][ty].g[tempidx] = circuit8.screen.screen[tx][ty+1].g[tempidx]; // scrolls up
						circuit8.screen.screen[tx][ty].b[tempidx] = circuit8.screen.screen[tx][ty+1].b[tempidx]; // scrolls up
					}
				}
				for (int tx = 0; tx < 64; tx++) {
					 circuit8.screen.screen[tx][48 - 1].r[tempidx] = temprowr[tx]; // top row to bottom
					 circuit8.screen.screen[tx][48 - 1].g[tempidx] = temprowg[tx]; // 
					 circuit8.screen.screen[tx][48 - 1].b[tempidx] = temprowb[tx]; // 
				}
			}
			circuit8.upscrolls++;
		} else if (circuit8.upscrolls >= 4 && circuit8.screenidx == 0) {
			float temprowr[48];
			float temprowg[48];
			float temprowb[48];
			for (int tempidx = 0; tempidx < 1024; tempidx++) {
				for (int tx = 0; tx < 48; tx++) {
					temprowr[tx] = circuit8.screen.screen[1][tx].r[tempidx]; // scrolls right
					temprowg[tx] = circuit8.screen.screen[1][tx].g[tempidx]; // scrolls right
					temprowb[tx] = circuit8.screen.screen[1][tx].b[tempidx]; // scrolls right
				}
				for (int tx = 0; tx < 48; tx++) {
					for (int ty = 0; ty < 64 - 1; ty++) {
						circuit8.screen.screen[ty][tx].r[tempidx] = circuit8.screen.screen[ty+1][tx].r[tempidx]; // scrolls right
						circuit8.screen.screen[ty][tx].g[tempidx] = circuit8.screen.screen[ty+1][tx].g[tempidx]; // scrolls right
						circuit8.screen.screen[ty][tx].b[tempidx] = circuit8.screen.screen[ty+1][tx].b[tempidx]; // scrolls right
					}
				}
				for (int tx = 0; tx < 48; tx++) {
					 circuit8.screen.screen[64 - 1][tx].r[tempidx] = temprowr[tx]; // right col to left
					 circuit8.screen.screen[64 - 1][tx].g[tempidx] = temprowg[tx]; // 
					 circuit8.screen.screen[64 - 1][tx].b[tempidx] = temprowb[tx]; // 
				}
			}
			circuit8.upscrolls = 0;
		}




//		cout << "screenidx updated" << endl;
//	cout << "DFDFF"<<endl;
	// kill half?
		killHalf();

//		cout << "expire" << endl;
	// Remove food
		expireFood();

//		cout << "inserting food" << endl;

	// Autoinsert Food
		autoinsertFood();

//		cout << "done inserting food" << endl;

	// remove all dead critters
		expireCritters();
		
//		cout << "expire critters" << endl;

	// Autosave Critters?
		autosaveCritters();

//		cout << "autoexchange" << endl;
	// Autoexchange Critters?
		autoexchangeCritters();

	// Autoinsert Critters?
		autoinsertCritters();

	// do a bullet step
		m_dynamicsWorld->stepSimulation(Timer::Instance()->bullet_ms / 1000.f);

//	cout << "render vision" << endl;

	renderVision();
	
//	cout << "render vision done" << endl;	
	
	grabVision();


//	cout << "procewss critters" << endl;
	
	// process all critters
	unsigned int lmax = cRitters.size();
	for( unsigned int i=0; i < lmax; i++)
	{
//		cout << "make a critter" << endl;
		CritterB *c = cRitters[i];

		if (critterselection->selectedCritter != c) { // does this just compare the pointers' values? or the whole objects
			c->lockblink = false;
			c->lockblinkscreen = false; 
			c->brain.lockdruglevels = false;
		}

		// TOUCH inputs and references -> find overlappings
			checkCollisions(  c );

//			cout << "getting group screen" << endl;
			getGroupScreen(c);
//			cout << "got group screen" << endl;

//	cout << "process one" << endl;
		// process
			c->process();

		// record critter used energy
			freeEnergy += c->energyUsed;

//	cout << "eat" << endl;
		// process Output Neurons
			eat(c);

//	cout << "procreate " << endl;
		// procreation if procreation energy trigger is hit
			procreate(c);

//			cout << "writing to group screen" << endl;
			alterGroupScreen(c);
//			cout << "written to group screen" << endl;

	}
//	cout << "end" << endl;

}

void WorldB::getGroupScreen(CritterB* c)
{
	// Read pixels into retina
	if ( !c->blinkscreen && !c->lockblinkscreen && cRitters.size() > 0 )
	{
		for (int i = 0;i<64;i++) { 
			for (int p = 0;p<48;p+=3) {
//					cout << i << " " << p << endl;
//					cout << circuit8.screen.screen[i][p].g[circuit8.screenidx];
					c->brain.Inputs[100000+(i*48)+p].output = circuit8.screen.screen[i][p].r[circuit8.screenidx];
					c->brain.Inputs[100000+(i*48)+p+1].output = circuit8.screen.screen[i][p].g[circuit8.screenidx];
					c->brain.Inputs[100000+(i*48)+p+2].output = circuit8.screen.screen[i][p].b[circuit8.screenidx];
//					c->brain.Inputs[100000+(i*600)+p].output = 1;
			}
		}
	}
}

void WorldB::alterGroupScreen(CritterB* c)
{
//	cout << circuit8.screenidx << " " << c->totalFrames << endl;
//	if (c->totalFrames < 3) { return; } // too noob
	// Read pixels into retina
	if ( !c->blinkscreen && !c->lockblinkscreen && cRitters.size() > 0 ) // could allow writes from critters with screen blinked
	{								     // i dont though
		int intintensity = 1;
		for (unsigned int i = 0;i<3;i++) {
			if (c->writeIntensity[i]) { intintensity = intintensity << 1; } // make the screen 64x64 and use shifts all the time
		}

//		if (c->totalFrames < 250) cout << "new critters write intensity is "<<intintensity<<endl; // get rid of this line

		float intensity;
		intensity = (float) intintensity / 10.0f;

		if (!*condensed_colourmotors) {
			for (int i = 0;i<64;i++) { 
				for (int p = 0;p<48;p++) {
/*					if (c->lessRed[(i*48)+p]) {      circuit8.screen.screen[i][p].r[circuit8.screenidx] -= intensity; if (circuit8.screen.screen[i][p].r[circuit8.screenidx] < 0) circuit8.screen.screen[i][p].r[circuit8.screenidx] = 0; }
					if (c->lessGreen[(i*48)+p]) {    circuit8.screen.screen[i][p].g[circuit8.screenidx] -= intensity; if (circuit8.screen.screen[i][p].g[circuit8.screenidx] < 0) circuit8.screen.screen[i][p].g[circuit8.screenidx] = 0; }
					if (c->lessBlue[(i*48)+p]) {     circuit8.screen.screen[i][p].b[circuit8.screenidx] -= intensity; if (circuit8.screen.screen[i][p].b[circuit8.screenidx] < 0) circuit8.screen.screen[i][p].b[circuit8.screenidx] = 0; }
					if (c->moreRed[(i*48)+p]) {      circuit8.screen.screen[i][p].r[circuit8.screenidx] += intensity; if (circuit8.screen.screen[i][p].r[circuit8.screenidx] > 1.0f) circuit8.screen.screen[i][p].r[circuit8.screenidx] = 1.0f; }
					if (c->moreGreen[(i*48)+p]) {    circuit8.screen.screen[i][p].g[circuit8.screenidx] += intensity; if (circuit8.screen.screen[i][p].g[circuit8.screenidx] > 1.0f) circuit8.screen.screen[i][p].g[circuit8.screenidx] = 1.0f; }
					if (c->moreBlue[(i*48)+p]) {     circuit8.screen.screen[i][p].b[circuit8.screenidx] += intensity; if (circuit8.screen.screen[i][p].b[circuit8.screenidx] > 1.0f) circuit8.screen.screen[i][p].b[circuit8.screenidx] = 1.0f; } */

					if (c->lessRed[(i*48)+p]) {      circuit8.lessRed(i,p,intensity); }
					if (c->lessGreen[(i*48)+p]) {    circuit8.lessGreen(i,p,intensity); }
					if (c->lessBlue[(i*48)+p]) {     circuit8.lessBlue(i,p,intensity); }
					if (c->moreRed[(i*48)+p]) {      circuit8.moreRed(i,p,intensity); }
					if (c->moreGreen[(i*48)+p]) {    circuit8.moreGreen(i,p,intensity); }
					if (c->moreBlue[(i*48)+p]) {     circuit8.moreBlue(i,p,intensity); }
				}
			}
		} else {
			for (int i = 0;i<64;i++) { 
				for (int p = 0;p<48;p++) {
					if (c->condensedColourOp[(i*48)+p]) {
						if (c->condensedLessRed && !c->condensedMoreRed)    {      circuit8.lessRed(i,p,intensity); }
						if (c->condensedLessGreen && !c->condensedMoreGreen)    {    circuit8.lessGreen(i,p,intensity); }
						if (c->condensedLessBlue && !c->condensedMoreBlue)    {     circuit8.lessBlue(i,p,intensity); }
						if (c->condensedMoreRed && !c->condensedLessRed)    {      circuit8.moreRed(i,p,intensity); }
						if (c->condensedMoreGreen && !c->condensedLessGreen)    {    circuit8.moreGreen(i,p,intensity); }
						if (c->condensedMoreBlue && !c->condensedLessBlue)    {     circuit8.moreBlue(i,p,intensity); }
					}
				}
			}
		}
	}
}



void WorldB::procreate( CritterB* c )
{
	if ( c->procreate && c->canProcreate )
	{
		bool brainmutant = false;
		bool bodymutant = false;
		if ( randgen->Instance()->get(1,100) <= settings->getCVar("brain_mutationrate") )
			brainmutant = true;

		if ( randgen->Instance()->get(1,100) <= settings->getCVar("body_mutationrate") )
			bodymutant = true;

		btDefaultMotionState* myMotionState = (btDefaultMotionState*)c->body.bodyparts[0]->body->getMotionState();
		btVector3 np = myMotionState->m_graphicsWorldTrans.getOrigin();

		// position offset
		np.setY(insertHight);
// 		if ( np.getX() > settings->getCVar("worldsizeX")/2 )
// 			np.setX(np.getX()-1.0f);
// 		else
// 			np.setX(np.getX()+1.0f);

		CritterB *nc = new CritterB(*c, currentCritterID++, np, brainmutant, bodymutant, &circuit8);
		// nc->circuit8 = &circuit8;
		//CritterB *nc = new CritterB(*c, currentCritterID++, findPosition(), mutant);

		// display message of birth
			stringstream buf;
			buf << setw(4) << c->critterID << " : " << setw(4) << nc->critterID;
			buf << " ad: " << setw(4) << nc->adamdist;
			buf << " n: " << setw(4) << nc->brain.totalNeurons << " s: " << setw(5) << nc->brain.totalSynapses;

			if ( brainmutant || bodymutant )
			{
				buf << " ";
				if ( brainmutant ) buf << "brain";
				if ( brainmutant && bodymutant ) buf << "+";
				if ( bodymutant ) buf << "body";
				buf << " mutant";
			}

			Textverbosemessage::Instance()->addBirth(buf);
			if (settings->getCVar("headless"))
				cerr << buf.str()<< endl;

		// split energies in half
			nc->energyLevel = c->energyLevel/2.0f;
			c->energyLevel = nc->energyLevel;

		// reset procreation energy count
			cRitters.push_back( nc );
			nc->calcFramePos(cRitters.size()-1);
	}
}

void WorldB::eat( CritterB* c )
{
	if ( c->eat )
	{
		if ( c->touchingFood )
		{
			Food* f = static_cast<Food*>(c->touchedEntity);
			
			float eaten = *critter_maxenergy / 100.0f;
			if ( c->energyLevel + eaten > *critter_maxenergy )
				eaten -= (c->energyLevel + eaten) - *critter_maxenergy;
			if ( f->energyLevel - eaten < 0 )
				eaten = f->energyLevel;

			c->energyLevel += eaten;
			f->energyLevel -= eaten;
		}
		else if ( c->touchingPill )
		{
//			cout << "DFSFSDFSDFSDF" << endl;
			Pill* p = static_cast<Pill*>(c->touchedEntity);
//			p->energyLevel = 10000.0f;

//			float eaten = 100.0f;
			float eaten = *pill_maxtrip / 100.0f;
//			cout << "DFSFSDFSDFSDF" << endl;
			if ( c->brain.tripping[p->molecule] + eaten > *pill_maxtrip ) { eaten -= (c->brain.tripping[p->molecule] + eaten) - *pill_maxtrip; }
//			cout << "DFSFSDFSDFSDF" << endl;
			if ( p->energyLevel - eaten < 0.0f) {
//				cout << "FGUCUCUCUSDUFSUFUSFUDUVCXUDFVUXDFUSDUFUSDFUSDF" << endl;
				eaten = p->energyLevel; }

//			cout << "DFSFSDFSDFSDF" << endl;
			c->brain.tripping[p->molecule] += eaten;
			p->energyLevel -= eaten;
//			cout << "DFSFSDFSDFSDF" << endl;
			if (p->energyLevel < 0.0f) { p->energyLevel = 0.0f; }
		}
		else if ( c->touchingCritter && settings->getCVar("critter_enableomnivores") )
		{
			CritterB* ct = static_cast<CritterB*>(c->touchedEntity);
			float eaten = *critter_maxenergy / 1000.0f;

                        float inthalfeaten = (float) (eaten / 2.0f); // to remove and add half
			//int inteaten = inthalfeaten * 2; // eaten critters energy               
                                                
			if ( c->energyLevel + eaten > *critter_maxenergy ) {
				eaten -= (c->energyLevel + eaten) - *critter_maxenergy; // hack
				inthalfeaten -= (c->energyLevel + inthalfeaten) - *critter_maxenergy; // hack
				//inteaten = inthalfeaten * 2; // hack for getting half of food energy still      
			}
			
			if ( ct->energyLevel - eaten < 0 ) { // eaten should always be >= inteaten so no need to check that too
				eaten = ct->energyLevel;
                       		inthalfeaten = (float) (eaten / 2.0f); // to remove and add half
                      		//inteaten = inthalfeaten * 2; // eaten critters energy    
			}

			if (ct->energyLevel >= 2) {
				freeEnergy += inthalfeaten; //  release half of carnivorous energy
        	                //  receive half of carnivorous energy
                	        c->energyLevel += inthalfeaten;
                	        ct->energyLevel -= inthalfeaten*2.0f;
				ct->eaten = true;
			} else {
				c->energyLevel += eaten; // if energy is low
				ct->energyLevel -= eaten;     // give all to eater
				ct->eaten = true;
			}	
		}
	}
//	cerr << "done eat" << endl;
}

void WorldB::killHalf()
{
	if ( cRitters.size() >= *critter_killhalfat )
	{
		killHalfOfCritters();
		
		// reduce energy :)
		if ( settings->getCVar("killhalfdecreaseenergybypct") > 0 )
		{
			float dec_amount = ((*food_maxenergy * *energy) - *food_maxenergy) / *food_maxenergy;
			if ( dec_amount >= 0.0f )
			{
				int dec = (dec_amount / 100) * settings->getCVar("killhalfdecreaseenergybypct");
				settings->setCVar("energy", settings->getCVar("energy")-dec );
				//*energy -= dec;
				freeEnergy -= dec * *food_maxenergy;
			}
		}
	}
}

void WorldB::autoinsertCritters()
{
	// insert critter if < minimum
	if ( cRitters.size() < settings->getCVar("mincritters") )
		insertCritter();

	// insert critter if insertcritterevery is reached
	if ( settings->getCVar("insertcritterevery") > 0 )
	{
		if ( insertCritterCounter >= settings->getCVar("insertcritterevery") )
		{
			insertCritter();
			insertCritterCounter = 0;
		}
		else
		{
			insertCritterCounter++;
		}
	}
}

void WorldB::autosaveCritters()
{
	if ( *critter_autosaveinterval > 0 )
	{
		autosaveCounter += ((float)Timer::Instance()->elapsed/1000);
		if ( autosaveCounter > *critter_autosaveinterval )
		{
			autosaveCounter = 0.0f;
			saveAllCritters();
		}
	}
}

void WorldB::autoexchangeCritters()
{
	if ( *critter_autoexchangeinterval > 0 )
	{
		autoexchangeCounter += ((float)Timer::Instance()->elapsed/1000);
		if ( autoexchangeCounter > *critter_autoexchangeinterval )
		{
			autoexchangeCounter = 0.0f;

			// determine exchange directory
			stringstream buf;
			buf << savedir << "/exchange";
			string exchangedir = buf.str();
			
			// save or load? :)
			unsigned int mode = randgen->Instance()->get( 0, 1 );
			
			// always load if critters == 0
			if ( cRitters.size() == 0 || mode )
			{
				vector<string> files;
				dirH.listContentsFull(exchangedir, files);

				// FIXME same routine as load all critters
				for ( unsigned int i = 0; i < files.size(); i++ )
				{
					if ( parseH->Instance()->endMatches( ".cr", files[i] ) )
					{
						stringstream buf;
						buf << "loading " << files[i];
						Logbuffer::Instance()->add(buf);

						string content;
						fileH.open( files[i], content ); 
						fileH.rm( files[i] );

						CritterB *c = new CritterB(content, m_dynamicsWorld, findPosition(), retina, &circuit8);
//						c->circuit8 = &circuit8;

						unsigned int error = 0;
						if ( c->retinasize != *critter_retinasize ) error = 1;

						if ( !error)
						{
							cRitters.push_back( c );

							c->critterID = currentCritterID++;
							c->calcFramePos(cRitters.size()-1);

							// start energy
							freeEnergy -= c->energyLevel;
						}
						else
						{
							delete c;
							if ( error == 1 )
							{
								stringstream buf;
								buf << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << files[i];
								Logbuffer::Instance()->add(buf);

								cerr << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << endl;
							}
						}
					}
				}
				stringstream buf;
				buf << "Loaded critters from " << exchangedir;
				Logbuffer::Instance()->add(buf);
				if (settings->getCVar("headless"))
					cerr << buf.str()<< endl;
			}
			else
			{
				// pick one to save
				unsigned int savec = randgen->Instance()->get( 0, cRitters.size()-1 );

				buf << "/" << time(0) << ".cr";
				string filename = buf.str();

				// makde dirs
				if ( !dirH.exists(savedir) )		dirH.make(savedir);
				if ( !dirH.exists(exchangedir) )	dirH.make(exchangedir);

				// save critter
				fileH.save(filename, cRitters[savec]->saveCritterB());

				//cerr << endl << "Saved critters to " << subsavedir << endl << endl;
				stringstream buf2;
				buf2 << "Autoexchange: Saved critter to " << filename;
				Logbuffer::Instance()->add(buf2);
				if (settings->getCVar("headless"))
					cerr << buf2.str()<< endl;
			}
		}
	}
}


void WorldB::autoinsertFood()
{
	if ( freeEnergy >= *food_maxenergy )
	{
		insertRandomFood(1, *food_maxenergy);
		freeEnergy -= *food_maxenergy;
		//cerr << "food: " << food.size() << endl;
		if (randgen->Instance()->get(1,1) == 1 && pillsinworld < 24) { WorldB::insertNextPill(1); }
		
	}
}




void WorldB::expireCritters()
{
	for( unsigned int i=0; i < cRitters.size(); i++)
	{
		// see if energy level isn't below 0 -> die, or die of old age
		if ( cRitters[i]->energyLevel <= 0.0f )
		{
			stringstream buf;
			if ( cRitters[i]->eaten )
				buf << setw(4) << cRitters[i]->critterID << " got eaten";
			else
				buf << setw(4) << cRitters[i]->critterID << " starved";
			Textverbosemessage::Instance()->addDeath(buf);
			if (settings->getCVar("headless"))
				cerr << buf.str()<< endl;

			removeCritter(i);
			i--;
		}
		// die of old age
		else if ( cRitters[i]->totalFrames > *critter_maxlifetime )
		{
			stringstream buf;
			buf << setw(4) << cRitters[i]->critterID << " died of age";
			Textverbosemessage::Instance()->addDeath(buf);
			if (settings->getCVar("headless"))
				cerr << buf.str()<< endl;

			removeCritter(i);
			i--;
		}
		// die if y < 100
		else
		{
			btDefaultMotionState* myMotionState = (btDefaultMotionState*)cRitters[i]->body.bodyparts[0]->body->getMotionState();
			btVector3 pos = myMotionState->m_graphicsWorldTrans.getOrigin();
			
			if ( pos.getY() < -100.0f )
			{
				stringstream buf;
				buf << setw(4) << cRitters[i]->critterID << " fell in the pit";
				Textverbosemessage::Instance()->addDeath(buf);
				if (settings->getCVar("headless"))
					cerr << buf.str()<< endl;

				removeCritter(i);
				i--;
			}
		}
	}
}

void WorldB::expireFood()
{
	for( unsigned int i=0; i < food.size(); i++)
	{
		// food was eaten
		if ( food[i]->energyLevel <= 0 )
		{
			freeEnergy += food[i]->energyLevel;
			if ( food[i]->isPicked )
				mousepicker->detach();
//			cout << "delete food one" << endl;
			delete food[i];
//			cout << "erase food one" << endl;
			food.erase(food.begin()+i);
			i--;
		}
		// old food, this should remove stuff from corners
		else if ( ++food[i]->totalFrames >= *food_maxlifetime )
		{
			freeEnergy += food[i]->energyLevel; 
			if ( food[i]->isPicked )
				mousepicker->detach();
//			cout << "delete food two" << endl;
			delete food[i]; 
//			cout << "erase food two" << endl;
			food.erase(food.begin()+i);
			i--;
		}
	}
	for (unsigned int i = 0; i < pill.size(); i++) {
		// pill was eaten
		if ( pill[i]->energyLevel <= 0 )
		{
			if (pill[i]->isPicked) { // feed pill
				mousepicker->detach();
			}
//			cout <<"delete pill"<<endl;
			delete pill[i];
//			cout <<"erase pill"<<endl;
			pill.erase(pill.begin()+i);
//			cout <<"urine pill"<<endl;
			i--;
			pillsinworld--;
		}
		else if ( ++pill[i]->totalFrames >= *pill_maxlifetime )
		{
			//freeEnergy += food[i]->energyLevel; 
			if ( pill[i]->isPicked )
				mousepicker->detach();
//			cout << "delete food two" << endl;
			delete pill[i]; 
//			cout << "erase food two" << endl;
			pill.erase(pill.begin()+i);
			i--;
			pillsinworld--;
		}
	}

/*		// die if y < 100
		else
		{
			btDefaultMotionState* myMotionState = (btDefaultMotionState*)food[i]->body.bodyparts[0]->body->getMotionState();
			btVector3 pos = myMotionState->m_graphicsWorldTrans.getOrigin();

			if ( pos.getY() < -100.0f )
			{
				freeEnergy += food[i]->energyLevel;
				if ( food[i]->isPicked )
					mousepicker->detach();
				delete food[i];
				food.erase(food.begin()+i);
				i--;
			}
		}*/
//	}
}

void WorldB::getGeneralStats()
{
	settings->info_totalNeurons = 0;
	settings->info_totalSynapses = 0;
	settings->info_totalAdamDistance = 0;
	settings->info_totalBodyparts = 0;
	settings->info_totalWeight = 0;

	settings->info_critters = cRitters.size();
	settings->info_food = food.size();

	for( unsigned int i=0; i < cRitters.size(); i++)
	{
		settings->info_totalNeurons		+= cRitters[i]->brain.totalNeurons;
		settings->info_totalSynapses		+= cRitters[i]->brain.totalSynapses;
		settings->info_totalAdamDistance	+= cRitters[i]->adamdist;
		settings->info_totalBodyparts		+= cRitters[i]->body.bodyparts.size();
		settings->info_totalWeight		+= cRitters[i]->body.totalWeight;
	}

	statsBuffer->add();

}

void WorldB::checkCollisions( CritterB* c )
{
	// set inputs to false and recheck
		c->touchingFood = false;
		c->touchingPill = false;
		c->touchingCritter = false;

	if ( c->body.mouths.size() > 0 )
	{
		btBroadphasePairArray& pairArray = c->body.mouths[0]->ghostObject->getOverlappingPairCache()->getOverlappingPairArray();
		int numPairs = pairArray.size();

		for ( int i=0; i < numPairs; i++ )
		{
			manifoldArray.clear();

			const btBroadphasePair& pair = pairArray[i];

			//unless we manually perform collision detection on this pair, the contacts are in the dynamics world paircache:
			btBroadphasePair* collisionPair = m_dynamicsWorld->getPairCache()->findPair(pair.m_pProxy0,pair.m_pProxy1);
			if (!collisionPair)
				continue;

			if (collisionPair->m_algorithm)
				collisionPair->m_algorithm->getAllContactManifolds(manifoldArray);

			for ( int j = 0; j < manifoldArray.size(); j++ )
			{
				btPersistentManifold* manifold = manifoldArray[j];
				
				btCollisionObject* object1 = static_cast<btCollisionObject*>(manifold->getBody0());
				btCollisionObject* object2 = static_cast<btCollisionObject*>(manifold->getBody1());

				if ( object1->getUserPointer() == c && object2->getUserPointer() == c )
					continue;

				for ( int p = 0; p < manifold->getNumContacts(); p++ )
				{
					const btManifoldPoint &pt = manifold->getContactPoint(p);
					if ( pt.getDistance() < 0.0f )
					{
						void* Collidingobject;
						if ( object1->getUserPointer() != c && object1->getUserPointer() != 0 )
							Collidingobject = object1->getUserPointer();
						else if ( object2->getUserPointer() != c && object2->getUserPointer() != 0 )
							Collidingobject = object2->getUserPointer();
						else 
							continue;

						// Touching Food
						Entity* e = static_cast<Entity*>(Collidingobject);
						if ( e->type == 1 )
						{
// 							cerr << "touches food" << endl;
							c->touchingFood = true;
							c->touchedEntity = e;
							return;
						}
						// Touching Critter
						else if ( e->type == 0 )
						{
// 							cerr << "touches critter" << endl;
							c->touchingCritter = true;
							c->touchedEntity = e;
							return;
						}
						// Touching Pill
						else if ( e->type == 3)
						{
							c->touchingPill = true;
							c->touchedEntity = e;
							return;
						}
/*						// Touching Food
						Food* f = static_cast<Food*>(Collidingobject);
						if ( f )
						{
							if ( f->type == 1 )
							{
								stop = true;
								c->touchingFood = true;
								c->touchedFoodID = f;
							}
							else
							{
								// Touching Critter
								CritterB* b = static_cast<CritterB*>(Collidingobject);
								if ( b->type == 0 )
								{
									stop = true;
									c->touchingCritter = true;
									c->touchedCritterID = b;
								}
							}
						}*/
					}
				}
			}
		}
	}
}


void WorldB::insertCritter()
{
	CritterB *c = new CritterB(m_dynamicsWorld, currentCritterID++, findPosition(), retina, &circuit8);
//	c->circuit8 = &circuit8;	
	cRitters.push_back( c );
	c->calcFramePos(cRitters.size()-1);

	// start energy
	freeEnergy -= c->energyLevel;
}

btVector3 WorldB::findPosition()
{
	return btVector3( (float)randgen->Instance()->get( 0, 100*settings->getCVar("worldsizeX") ) / 100, insertHight, (float)randgen->Instance()->get( 0, 100*settings->getCVar("worldsizeY") ) / 100 );
}

void WorldB::removeCritter(unsigned int cid)
{
	freeEnergy += cRitters[cid]->energyLevel;

	if ( cRitters[cid]->isPicked )
		mousepicker->detach();
	
	critterselection->unregisterCritterID(cRitters[cid]->critterID);
	critterselection->deselectCritter(cRitters[cid]->critterID);

	delete cRitters[cid];
	cRitters.erase(cRitters.begin()+cid);

	// update higher retina frame positions
	for ( unsigned int c = cid; c < cRitters.size(); c++ )
		cRitters[c]->calcFramePos(c);
}

void WorldB::killHalfOfCritters()
{
	for ( unsigned int c = 0; c < cRitters.size(); c++ )
		removeCritter(c);
//	removeCritter(0); // just kill oldest hack
}

void WorldB::renderVision()
{
	// render critter vision
	if ( !*critter_raycastvision )
		for( unsigned int i=0; i < cRitters.size(); i++)
			if ( cRitters[i]->body.mouths.size() > 0 )
			{
				cRitters[i]->place();
//				cout << "aaaaaaa" << endl;
				drawWithinCritterSight(cRitters[i]);
			}
}

void WorldB::grabVision()
{
	// Read pixels into retina
	if ( !*critter_raycastvision && cRitters.size() > 0 )
	{
		// determine height
		unsigned int picheight = *critter_retinasize;
		unsigned int rows = cRitters.size();
		while ( rows > *retinasperrow )
		{
			picheight += *critter_retinasize;
			rows -= *retinasperrow;
		}
		glReadBuffer(GL_BACK);
		glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
		glReadPixels(0, 0, picwidth, picheight, GL_RGBA, GL_UNSIGNED_BYTE, retina);
	}
}


void WorldB::drawWithoutFaces()
{
//	cout << "wtf" << endl;
 	for( unsigned int i=0; i < cRitters.size(); i++)
		cRitters[i]->draw(false);

//	cout << "WTF" << endl;
	for( unsigned int i=0; i < pill.size(); i++)
		pill[i]->draw();

	for( unsigned int i=0; i < food.size(); i++)
		food[i]->draw();

	for( unsigned int i=0; i < walls.size(); i++)
		walls[i]->draw();

	// draw floor
// 	grid.draw();
}


void WorldB::drawWithGrid()
{
//	cout << "tfw" << endl;
	for( unsigned int i=0; i < cRitters.size(); i++)
		cRitters[i]->draw(true);

	for( unsigned int i=0; i < food.size(); i++)
		food[i]->draw();

//	printf("%i\n",pill.size());
	for( unsigned int i=0; i < pill.size(); i++)
		pill[i]->draw();

//	printf("aaaaaaa\n");
	for( unsigned int i=0; i < walls.size(); i++)
		walls[i]->draw();

//	printf("aaaaaaa\n");
// 	m_dynamicsWorld->debugDrawWorld();

//	printf("sdfsdf\n");

	// draw floor
// 	grid.draw();
}


void WorldB::drawWithinCritterSight(CritterB *c)
{
// 	CritterB *c = critters[cid];

	if (!c->blink && !c->lockblink) {
		if ( c->body.mouths.size() > 0 )
		{
			btDefaultMotionState* cmyMotionState = (btDefaultMotionState*)c->body.mouths[0]->body->getMotionState();
			btVector3 cposi = cmyMotionState->m_graphicsWorldTrans.getOrigin();

			// draw everything in it's sight
			float sightrange = (float)*critter_sightrange/10;

			btScalar position[16];

			//		cout << "asdf" << endl;

			for( unsigned int i=0; i < walls.size(); i++)
				walls[i]->draw();

			for( unsigned int i=0; i < food.size(); i++)
			{
				Food *f = food[i];
				if ( cposi.distance( f->myMotionState->m_graphicsWorldTrans.getOrigin() ) < sightrange )
					f->draw();
			}

			for( unsigned int i=0; i < pill.size(); i++)
			{
				Pill *p = pill[i];
				if ( cposi.distance( p->myMotionState->m_graphicsWorldTrans.getOrigin() ) < sightrange )
					p->draw();
					}


			for( unsigned int j=0; j < cRitters.size(); j++)
			{
				CritterB *f = cRitters[j];
				for( unsigned int b=0; b < f->body.bodyparts.size(); b++)
				{
					const btDefaultMotionState* fmyMotionState = static_cast<const btDefaultMotionState*>(f->body.bodyparts[b]->body->getMotionState());
					if ( cposi.distance( fmyMotionState->m_graphicsWorldTrans.getOrigin() ) < sightrange )
					{
						fmyMotionState->m_graphicsWorldTrans.getOpenGLMatrix(position);
						glPushMatrix(); 
						glMultMatrixf(position);
						
								glColor4f( f->color[0], f->color[1], f->color[2], 0.0f );

								btVector3 halfExtent = static_cast<const btBoxShape*>(f->body.bodyparts[b]->shape)->getHalfExtentsWithMargin();
								glScaled(halfExtent[0], halfExtent[1], halfExtent[2]);
								
								Displaylists::Instance()->call(0);

						glPopMatrix();
					}
				}
				if ( c->critterID != cRitters[j]->critterID )
				{
					for( unsigned int j=0; j < f->body.mouths.size(); j++)
					{
						if ( cposi.distance( f->body.mouths[j]->ghostObject->getWorldTransform().getOrigin() ) < sightrange )
						{
							f->body.mouths[j]->ghostObject->getWorldTransform().getOpenGLMatrix(position);
						
							glPushMatrix(); 
							glMultMatrixf(position);

									glColor4f( 1.0f, 0.0f, 0.0f, 0.0f );

									btVector3 halfExtent = static_cast<const btBoxShape*>(f->body.mouths[j]->shape)->getHalfExtentsWithMargin();
									glScaled(halfExtent[0], halfExtent[1], halfExtent[2]);

									Displaylists::Instance()->call(1);
								
							glPopMatrix();
						}
					}
				}
			}
		}
	} //else { cout << "blinked" << endl; }
}

void WorldB::loadAllCritters()
{
	vector<string> files;
	dirH.listContentsFull(loaddir, files);

	for ( unsigned int i = 0; i < files.size(); i++ )
	{
		if ( parseH->Instance()->endMatches( ".cr", files[i] ) )
		{
			stringstream buf;
			buf << "loading " << files[i];
			Logbuffer::Instance()->add(buf);

			string content;
			fileH.open( files[i], content ); 

			cout << "loading " << files[i] << endl;
			CritterB *c = new CritterB(content, m_dynamicsWorld, findPosition(), retina, &circuit8);
//			c->circuit8 = &circuit8;
			cout << "loaded" << endl;

			unsigned int error = 0;
			if ( c->retinasize != *critter_retinasize ) error = 1;

			if ( !error)
			{
				cRitters.push_back( c );

				c->critterID = currentCritterID++;
				c->calcFramePos(cRitters.size()-1);

				// start energy
				freeEnergy -= c->energyLevel;
				// c->brain.tripping = 0.0f; // should all start 0
			}
			else
			{
				delete c;
				if ( error == 1 )
				{
					stringstream buf;
					buf << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << files[i];
					Logbuffer::Instance()->add(buf);

					cerr << "ERROR: critter retinasize (" << c->retinasize << ") doesn't fit world retinasize (" << *critter_retinasize << ")" << endl;
				}
			}
		}
	}
	stringstream buf;
	buf << "Loaded critters from " << loaddir;
	Logbuffer::Instance()->add(buf);
	//cerr << endl << "Loaded critters from " << loaddir << endl << endl;
}

void WorldB::saveAllCritters()
{
	// determine save directory
	stringstream buf;
	buf << savedir << "/" << settings->profileName;
	string subprofiledir = buf.str();
	
	buf << "/" << time(0);
	string subsavedir = buf.str();

	// makde dirs
	if ( !dirH.exists(savedir) )		dirH.make(savedir);
	if ( !dirH.exists(subprofiledir) )	dirH.make(subprofiledir);
	if ( !dirH.exists(subsavedir) )		dirH.make(subsavedir);

	for ( unsigned int i = 0; i < cRitters.size(); i++ )
	{
		// determine filename
		stringstream filename;
		filename << subsavedir << "/" << "critter" << i << ".cr";
	
		// save critters
		fileH.save(filename.str(), cRitters[i]->saveCritterB());
	}
 	//cerr << endl << "Saved critters to " << subsavedir << endl << endl;
	stringstream buf2;
	buf2 << "Saved critters to " << subsavedir;
	Logbuffer::Instance()->add(buf2);

}

void WorldB::tripfirst() {
//	critters[0]->brain.tripping[0] += 100.0;
//	critters[0]->brain.tripping[1] += 100.0;
//	critters[0]->brain.tripping[2] += 100.0;
//	critters[0]->brain.tripping[3] += 100.0;

//	cRitters[0]->brain.tripping[randgen->Instance()->get(0,4)] += 100.0;

	if      (circuit8.screendirection == 0) { circuit8.screendirection = 2; }
	else if (circuit8.screendirection == 1) { circuit8.screendirection = 3; }
	else if (circuit8.screendirection == 2) { circuit8.screendirection = 0; }
	else if (circuit8.screendirection == 3) { circuit8.screendirection = 1; }
}

void WorldB::tripthis1() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[0] += 1000;
        	if (critterselection->selectedCritter->brain.tripping[0] > *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[0] = *pill_maxtrip;
        	}
        }
}

void WorldB::tripthis2() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[1] += 1000;
        	if (critterselection->selectedCritter->brain.tripping[1] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[1] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::tripthis3() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[2] += 1000;
        	if (critterselection->selectedCritter->brain.tripping[2] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[2] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::tripthis4() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[3] += 1000;
        	if (critterselection->selectedCritter->brain.tripping[3] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[3] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::tripthis5() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	critterselection->selectedCritter->brain.tripping[4] += 1000;
        	if (critterselection->selectedCritter->brain.tripping[4] > (float) *pill_maxtrip) {
        		critterselection->selectedCritter->brain.tripping[4] = (float) *pill_maxtrip;
        	}
        }	
}

void WorldB::feedthis1() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	float feeden = 0;
        	if (critterselection->selectedCritter->energyLevel + *food_maxenergy > *critter_maxenergy) {
        		feeden = *critter_maxenergy - critterselection->selectedCritter->energyLevel;
        	} else {
        		feeden = *food_maxenergy;
        	}
        	critterselection->selectedCritter->energyLevel += feeden;
		freeEnergy -= feeden;
	}
}

void WorldB::togglelockblink1() {
        if ( critterselection->selectedCritter == 0 )
        {
        	cout << "lockblink1 returned" << endl;
        	return;
	}
	else
        {
        	if (critterselection->selectedCritter->lockblink) {
//        		cout << "lockblink1 set to false" << endl;
        		critterselection->selectedCritter->lockblink = false;
		} else {
//			cout << "lockblink1 set to true" << endl;
			critterselection->selectedCritter->lockblink = true;
		}
	}	
}

void WorldB::togglelockblink2() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	if (critterselection->selectedCritter->lockblinkscreen) { critterselection->selectedCritter->lockblinkscreen = false; } else { critterselection->selectedCritter->lockblinkscreen = true; }
	}	
}

void WorldB::togglelockdruglevels() {
        if ( critterselection->selectedCritter == 0 )
        {
        	return;
	}
	else
        {
        	if (critterselection->selectedCritter->brain.lockdruglevels) { critterselection->selectedCritter->brain.lockdruglevels = false; } else { critterselection->selectedCritter->brain.lockdruglevels = true; }
	}	
}


void WorldB::createDirs()
{
#ifndef _WIN32
 	homedir = getenv("HOME");
 	if ( homedir.empty() ) {
		cout << "environment variable HOME not defined/detected" << endl;
		exit(0);
	}
	progdir = homedir;	progdir.append("/.critterding");
	savedir = progdir;	savedir.append("/save");
	loaddir = progdir;	loaddir.append("/load");
#else
	char mydoc[256];
	memset(mydoc, 0, sizeof(mydoc));

	SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 0, mydoc);
	homedir.assign(mydoc);
	progdir = homedir;	progdir.append("\\critterding");
	savedir = progdir;	savedir.append("\\save");
	loaddir = progdir;	loaddir.append("\\load");
#endif

//	cerr << progdir << endl;

	if ( !dirH.exists(progdir) ) dirH.make(progdir);
	if ( !dirH.exists(loaddir) ) dirH.make(loaddir);
}

void WorldB::resetCamera()
{
	unsigned int biggest = settings->getCVar("worldsizeX");
	if ( settings->getCVar("worldsizeY") > biggest )
		biggest = 1.4f*settings->getCVar("worldsizeY");

	camera.position.setOrigin( btVector3( 0.5f*settings->getCVar("worldsizeX"), 1.3f*biggest, 0.5f*settings->getCVar("worldsizeY")) );
// 	camera.position.setRotation(btQuaternion(btVector3(1, 0, 0), btScalar(90)));
	camera.position.getBasis().setEulerZYX( -1.5707f, 0.0f, 0.0f ); // 1.5707f  (float)settings->getCVar("energy")/10


// 	camera.position = btVector3( -0.5f*settings->getCVar("worldsizeX"), -1.3f*biggest, -0.5f*settings->getCVar("worldsizeY"));
// 	camera.rotation = Vector3f( 90.0f,  0.0f, 0.0f);
}

void WorldB::makeFloor()
{
	makeDefaultFloor();
}

void WorldB::makeDefaultFloor()
{
	for ( unsigned int i=0; i < walls.size(); i++ )	
		delete walls[i];
	walls.clear();

	// Wall Constants
		float WallWidth = 0.5f;
		float WallHalfWidth = WallWidth/2.0f;
		float WallHeight = 2.0f;
		float WallHalfHeight = WallHeight/2.0f;

	// Ground Floor
		btVector3 position( settings->getCVar("worldsizeX")/2.0f, -WallHalfWidth, settings->getCVar("worldsizeY")/2.0f );
		Wall* w = new Wall( settings->getCVar("worldsizeX"), WallWidth, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
		w->color[0] = 0.30f; w->color[1] = 0.20f; w->color[2] = 0.10f;
		walls.push_back(w);
	
	if ( settings->getCVar("worldwalls") )
	{
		// Left Wall
			position = btVector3 ( 0.0f-WallHalfWidth, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")/2.0f );
			w = new Wall( WallWidth, WallHeight, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Right Wall
			position = btVector3 ( settings->getCVar("worldsizeX")+WallHalfWidth, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")/2.0f );
			w = new Wall( WallWidth, WallHeight, settings->getCVar("worldsizeY"), position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Top Wall
			position = btVector3 ( settings->getCVar("worldsizeX")/2.0f, WallHalfHeight-WallWidth, 0.0f-WallHalfWidth );
			w = new Wall( settings->getCVar("worldsizeX")+(WallWidth*2), WallHeight, WallWidth, position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
		// Bottom Wall
			position = btVector3 ( settings->getCVar("worldsizeX")/2.0f, WallHalfHeight-WallWidth, settings->getCVar("worldsizeY")+WallHalfWidth );
			w = new Wall( settings->getCVar("worldsizeX")+(WallWidth*2), WallHeight, WallWidth, position, m_dynamicsWorld );
			w->color[0] = 0.34f; w->color[1] = 0.25f; w->color[2] = 0.11f;
			walls.push_back(w);
	}
}

// camera ops
void WorldB::camera_moveup() { camera.moveUp(0.01f); movePickedBodyFrom(); }
void WorldB::camera_movedown() { camera.moveDown(0.01f); movePickedBodyFrom(); }
void WorldB::camera_moveforward() { camera.moveForward(0.01f); movePickedBodyFrom(); }
void WorldB::camera_movebackward() { camera.moveBackward(0.01f); movePickedBodyFrom(); }
void WorldB::camera_moveleft() { camera.moveLeft(0.01f); movePickedBodyFrom(); }
void WorldB::camera_moveright() { camera.moveRight(0.01f); movePickedBodyFrom(); }

void WorldB::camera_lookup() { camera.lookUp(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_lookdown() { camera.lookDown(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_lookleft() { camera.lookLeft(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_lookright() { camera.lookRight(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_rollleft() { camera.rollLeft(0.001f); calcMouseDirection(); movePickedBodyTo(); }
void WorldB::camera_rollright() { camera.rollRight(0.001f); calcMouseDirection(); movePickedBodyTo(); }

int WorldB::insertRandomFood(int amount, float energy)
{
	for ( int i=0; i < amount; i++ )
	{
		Food *f = new Food;
		f->energyLevel = energy;
		//f->resize();
		f->createBody( m_dynamicsWorld, findPosition() );
		
		food.push_back( f );
	}
}


void setPillColor(Pill* p) {
			if (p->molecule == 4) {            // potentiator purple or pink dunno
			        p->color[0] = 0.8f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.8f;
                                p->color[3] = 0.0f;
			} else if (p->molecule == 3) {            // excitatory agonist bright red
			        p->color[0] = 0.9f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.3f;
                                p->color[3] = 0.0f;
			} else if (p->molecule == 2) {     // excitatory antagonist dark red
			        p->color[0] = 0.3f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.05f;
                                p->color[3] = 0.0f;
			} else if (p->molecule == 1) {     // inhibitory agonist bright blue
			        p->color[0] = 0.2f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 1.0f;
                                p->color[3] = 0.0f;
			} else if (p->molecule == 0) {     // inhibitory antagonist dark blue
			        p->color[0] = 0.1f;
			        p->color[1] = 0.0f;
	                        p->color[2] = 0.5f;
                                p->color[3] = 0.0f;
			}
			return;
}

int WorldB::insertNextPill(int anything)
{
		if (pill.size() < 24) {
			Pill* p  = new Pill;
			p->energyLevel = *pill_energylevel;
			p->molecule = randgen->Instance()->get(0,4);
			setPillColor(p);
			//p->energyLevel = energy;
			//		p->resize(120);
			p->createBody( m_dynamicsWorld, findPosition() );
			pill.push_back( p );
			pillidx += 1;
			
		} else {
			Pill* p = pill[pillidx];
			p->energyLevel = *pill_energylevel;
			p->molecule = randgen->Instance()->get(0,4);
			setPillColor(p);
			pill[pillidx] = p;
			pillidx += 1;
		}
		pillidx += 1;
		if (pillidx > 24 - 1) {pillidx = 0;}
		pillsinworld += 1;
}


WorldB::~WorldB()
{
	for ( unsigned int i=0; i < food.size(); i++ )		delete food[i];
	for ( unsigned int i=0; i < pill.size(); i++ )		delete pill[i];
	for ( unsigned int i=0; i < cRitters.size(); i++ )	delete cRitters[i];
	for ( unsigned int i=0; i < walls.size(); i++ )		delete walls[i];

	free(retina);

	delete m_collisionConfiguration;
	delete m_dispatcher;
	delete m_broadphase;
	delete m_solver;
	delete m_dynamicsWorld;
	
	delete raycast;
	delete mousepicker;
}
